import os
import requests
import sys
import time

class Bot_Update:
    def __init__(self,repo_owner:str,repo_name:str,version:str,access_token:str) -> None:
        """_summary_
         Check for the updates in repo if there are updates, it will automatically updates the bot 

        Args:
            repo_owner  : Enter Your NAME 
            repo_name   : Enter Repo NAME
            version     : Enter the Bot Version
            access_toke : Enter the Access Token
        """
        
        self.REPO_OWNER = repo_owner
        self.REPO_NAME = repo_name
        self.CURRENT_VERSION = version
        self.ACCESS_TOKEN = access_token
        self.NEW_VERSION_FILE = ""
        folder = os.getcwd()
        files = [os.path.join(folder, f) for f in os.listdir(folder) if f.lower().endswith('.bat')]
        for f in files:
            os.remove(f) 
            time.sleep(2)
        print("Checking for updates...")
        download_url = self.check_for_update()
        if download_url:
            if self.download_update(download_url):
                print(f"{self.NEW_VERSION_FILE} Downloaded Successfully")
                time.sleep(3)
                self.self_delete()
                exit()
            else:
                print("Can't Download the bot")
        
    
    def check_for_update(self):
        
        """Check GitHub for the latest release."""
        url = f"https://api.github.com/repos/{self.REPO_OWNER}/{self.REPO_NAME}/releases/latest"
        headers = {
            "Authorization": f"token {self.ACCESS_TOKEN}",
            
            }
        # print(url)
        try:
            response = requests.get(url, headers=headers)
            if response.status_code == 200:
                release_info = response.json()
                # print(release_info)
                latest_version = release_info['tag_name']
                # download_url = release_info['assets'][0]['browser_download_url']
                download_url = release_info['assets'][0]['url']
                self.NEW_VERSION_FILE = release_info['assets'][0]['name']
                print(f"Latest version: {latest_version}, Current version: {self.CURRENT_VERSION}")
                if latest_version > self.CURRENT_VERSION:
                    print("Update available!")
                    return download_url
                else:
                    print("No update available.")
            else:
                print(f"Failed to fetch release info: {response.status_code}")
        except Exception as e:
            print(f"Error checking for update: {e}")
        return None

    def download_update(self,download_url):
        
        """Download the latest version."""
        print("Downloading update...")
        try:
            headers = {
            "Authorization": f"token {self.ACCESS_TOKEN}",
            "Accept": "application/octet-stream"
        }
        # Send a GET request to the file URL and follow redirects
            response = requests.get(download_url,headers=headers,stream=True,allow_redirects=True)
            # Check if the request was successful
            print(f'Response Code: {response.status_code}')
            if response.status_code == 200:
                # Open the file in write-binary mode and save the content
                with open(self.NEW_VERSION_FILE, 'wb') as file:
                    for chunk in response.iter_content(chunk_size=8192):  # Download in chunks
                        file.write(chunk)
                print(f"File downloaded successfully")
                return True
            else:
                print("Response code is not 200")
                return False
        except Exception as e:
            print(f"Error downloading update: {e}")
            return False

    def self_delete(self):
        # Get the current script path
        script_path = os.path.abspath(sys.argv[0])
     
        # Properly quote the path to handle spaces
        script_path_quoted = f'"{script_path}"'

        # Create a batch command to delete the script
        delete_command = fr'''
        @echo off
        timeout /t 2 >nul
        del {script_path_quoted}
        exit
        '''

        # Save the batch script in the same directory as the script
        batch_path = "deleter.bat"
        with open(batch_path, "w") as batch_file:
            batch_file.write(delete_command)
        
        # Execute the batch script to delete this script
        os.system(f'start /b {batch_path}')




class Maildealer:
    def __init__(self,maildealer_config):
        
        self.maildealer_config = maildealer_config

    
    def login(self,driver:object) -> None:
        """_summary_
        Used to Login to Maildealer 
        
        Args:
            driver : selenium.chrome.WebDriver
        """
        
        Maildealerurl = self.maildealer_config["MailDealer_url"]
        driver.get(Maildealerurl)
        
        logid = driver.find_element("xpath",self.maildealer_config["login_xpaths"]["ユーザID"])#ログインIDの要素読み込み
        logpassword = driver.find_element("xpath",self.maildealer_config["login_xpaths"]["パスワード"])#パスワードの要素読み込み

        logid.send_keys(self.maildealer_config["username"])
        logpassword.send_keys(self.maildealer_config["password"])

        # driver.find_element("xpath", '/html/body/div/div[1]/div[2]/div/form/div/input[2]')

        #Click on the Login button
        driver.find_element("xpath", self.maildealer_config["login_xpaths"]["ログイン"]).click()

        print("Login successful")
        



import time
from selenium.webdriver.common.by import By
import pyautogui
import pyperclip


class SharePoint:
    
    search = ""
    
    def __init__(self,sharepoint_config):
        """_summary_

        Args:
            sharepoint_config (json): Json config of sharepoint
        """
        self.sharepoint_config =  sharepoint_config
        
    
    def upload_folder(self,driver:object,url:str,folder_path:str):
        """_summary_

        Args:
            driver: selenium.chrome.WebDriver
            url : Url of the Sharepoint Folder 
            folder_path : Folder Path to be Uploaded 

        Returns:
            True if the folder Uploaded Succcessfully 
        """
        
        self.driver = driver
        
        self.driver.get(url)
        time.sleep(2)
        
        upload_button = self.driver.find_element(By.XPATH,self.sharepoint_config["xpaths"]["upload"])
        upload_button.click()
        time.sleep(1)
        
        folder_button = self.driver.find_element(By.XPATH,self.sharepoint_config["xpaths"]["folder"])
        folder_button.click()
        time.sleep(2)
        
        pyperclip.copy(folder_path)
        print("folder path is:", folder_path)
        pyautogui.hotkey('ctrl','v')
        print('ctrl v pressed')
        time.sleep(2)
        pyautogui.hotkey('enter')
        time.sleep(2)
        # pyautogui.click(645, 450)
        # pyautogui.hotkey('ctrl','a')
        # time.sleep(2)
        pyautogui.hotkey('enter')
        time.sleep(2)
        pyautogui.press('left')
        time.sleep(2)
        pyautogui.hotkey('enter')
        time.sleep(10)
        print("success")
        return True

        
    def handle_login(self,driver:object):
        """_summary_

        Args:
            driver : selenium.chrome.WebDriver
        """
        
        url=self.sharepoint_config["sharepoint_url"]
        
        self.driver = driver
        # Assuming the login page has input fields with IDs 'username' and 'password'
        self.driver.get(url)

        # username = "kushalnasiwak@nskkogyo.onmicrosoft.com"
        # password = "Vay32135"
        
        username = self.sharepoint_config["username"]
        password = self.sharepoint_config["password"]
        time.sleep(2.5)
        # Find the username input field on the login page
        email_field = self.driver.find_element(By.XPATH, self.sharepoint_config['login_xpaths']['email'])
        email_field.clear()
        email_field.send_keys(username)
        self.driver.find_element(By.XPATH, self.sharepoint_config['login_xpaths']['loggin_button']).click()
        time.sleep(1.5)
        password_field = self.driver.find_element(By.XPATH, self.sharepoint_config['login_xpaths']['password'])
        password_field.clear()
        password_field.send_keys(password)
        self.driver.find_element(By.XPATH, self.sharepoint_config['login_xpaths']['loggin_button']).click()
        time.sleep(1.5)
        
        self.driver.find_element(By.XPATH,self.sharepoint_config['login_xpaths']['checkbox']).click()
        self.driver.find_element(By.XPATH,self.sharepoint_config['login_xpaths']['yes_button']).click()
        time.sleep(2)
        
        print('Logged in to Sharepoint\nPlease wait.....')


    


import shutil
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
import requests
import os
import zipfile

def create_driver():
        
        """_summary_
        creates a selenium.chrome.WebDriver and returns it

        Returns:
            selenium.chrome.WebDriver : Chrome WebDrivere 
        """
        chrome_options = Options()
        # download_folder = os.path.join(os.getcwd(), f'{id_folder_name}')
        chrome_options.add_experimental_option("prefs", {
        #"download.default_directory": download_folder,
        "download.prompt_for_download": False,
        "download.directory_upgrade": True,
        "safebrowsing.enabled": True
        })

        # chrome_options.add_argument("--lang=en")

        # Initialize the WebDriver before the loop
        driver = webdriver.Chrome(options=chrome_options)
        driver.maximize_window()
        
        return driver
    

def create_json_config(url,ACCESS_TOKEN) -> dict: 
    """_summary_

    Args:
        url : git file row url
        ACCESS_TOKEN : your access token

    Returns:
        json response: Returns the json-encoded content of a response, if any.
    """
    headers = {"Authorization": f"token {ACCESS_TOKEN}"}
    response = requests.get(url, headers=headers)
    
    return response.json()

def extractZip(folder:str):
    """_summary_
    it will take the folder path of the zip file and extract all the zip files in that folder 
    and delete the extracted zip file 

    Args:
        folder : folder path of the zip file where zip file is present
    """
    zip_files = [os.path.join(folder, f) for f in os.listdir(folder) if f.lower().endswith('.zip')]
    for zip_file in zip_files:
        if zip_file.endswith('.zip'):
            with zipfile.ZipFile(zip_file, "r") as zf:
                for info in zf.infolist():
                    try:
                        info.filename = info.orig_filename.encode('cp932').decode('cp932')
                    except:
                        info.filename = info.orig_filename.encode('cp437').decode('cp932')
                    if os.sep != "/" and os.sep in info.filename:
                        info.filename = info.filename.replace(os.sep, "/")
                    zf.extract(info, path=folder)
            
            zip_folder = f'{zip_file}'.split('.zip')[0]
            # logging.info(zip_folder)
            for file in os.listdir(zip_folder):
                try:
                    shutil.move(zip_folder+rf'\{file}',folder)
                except:
                    pass
            shutil.rmtree(zip_folder)
            try:
                os.remove(zip_file)
            except Exception as E:
                print('While deleting zip file i got this error')
                print(f"{E}")
                
def folder_to_zip(folder_path:str, zipfile_path:str):
    """_summary_
        converts the given folder into zip file in the given path
        exaple values for reference\n
        folder_path = 'myfolder'\n
        zipfile_path = 'myfolder.zip'
    Args:
        folder_path (str): path of the folder you want to convert into zip 
        zipfile_path (str): path and name of the zip folder with .zip extenction 
    """

    with zipfile.ZipFile(zipfile_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
        for root, dirs, files in os.walk(folder_path):
            for file in files:
                # Create the full path of the file
                file_path = os.path.join(root, file)
                # Add file to the ZIP, preserving the folder structure
                arcname = os.path.relpath(file_path, folder_path)
                zipf.write(file_path, arcname)


import time
import pyautogui


class Aisys:
    def __init__(self,aisys_config) -> None:
        """_summary_
        
        Args:
            Aisys_config (json): Json config of aisys
        """
        self.Aisys_config = aisys_config
        
    def Aisys_login(self,driver):
        driver.get(self.Aisys_config['aisys_url'])
        
        #Log In to Aikomuten
        userId = self.Aisys_config['aisys_userid']
        id = driver.find_element('xpath',self.Aisys_config["login_xpaths"]["ログインID"])
        id.send_keys(userId)

        userPwd = self.Aisys_config['aisys_password']
        pwd = driver.find_element('xpath',self.Aisys_config["login_xpaths"]["パスワード"])
        pwd.send_keys(userPwd)

        print('Entered Username and Password')

        submit1 = driver.find_element('xpath',self.Aisys_config["login_xpaths"]["ログイン"])
        submit1.click()
        
        
        print('Login Successful to Ai-koumuten')
        time.sleep(2)
        pyautogui.press('Enter')
        
        
    
    


from .sharepoint import SharePoint
from .webaccess import Webaccess
from .creater import create_driver
from .creater import create_json_config
from .creater import extractZip
from .creater import folder_to_zip
from .maildealer import Maildealer
from .bot_update import Bot_Update
from .aisys import Aisys


from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC


class Webaccess:
    def __init__(self,webaccess_config) -> None:
        """_summary_

        Args:
            json_config (json): json config of webaccess
        """
        self.config = webaccess_config

    def WebAccess_login(self,driver:object,user_id:str = "NasiwakRobot",password:str = "159753"):
        
        
        """_summary_
        Used To login to WebAccess 
        Args:
            driver : selenium.chrome.WebDriver
            user_id : Userid of your account  default value -> "NasiwakRobot"
            password : Password of your account default value -> "159753"
        """
        
        # webaccess_url = 'https://webaccess.nsk-cad.com/order_list.php'
        webaccess_url = self.config["webaccess_url"]
        driver.get(webaccess_url)
        # user_id = "NasiwakRobot"
        # password = "159753"
        try:
            logid = driver.find_element("name","u")
            logpassword = driver.find_element("name","p")

            logid.clear()
            logpassword.clear()

            logid.send_keys(user_id)
            logpassword.send_keys(password)
            logid.submit()
            driver.implicitly_wait(10)

            wait = WebDriverWait(driver, 10)
            wait.until(EC.presence_of_all_elements_located)
            print("Successfully logged in to Webaccess")
        except:
            print('logged in already')
            



    
    


# setup.py
from setuptools import setup, find_packages

setup(
    name="Nasiwak",            # Your package name
    version="0.1.9",             # Package version
    author="Nasiwak Team",
    author_email="Kushalnasiwak@outlook.com",
    description="A model that contains all the needs of Nasiwak Company Developers",
    packages=find_packages(),    # Automatically discover all packages in the module
    install_requires=[
        "selenium >= 4.26.0",
        "pyautogui >= 0.9.54",
        ""
        ],         # Dependencies (add here if your code depends on other packages)
    license="MIT",
)